#pragma once
#include <iostream>
#include <map>
#include <time.h>
#define TMap std::map
#include "FBullCowGame.h"

FBullCowGame::FBullCowGame()
{
}

FString FBullCowGame::PickWordLength()
{
	FString SNum;
	int32 Num;
	std::cout << "Pick a number from 4 to 8." << std::endl;

	//loop until the user has picked a valid number.
	do
	{
		std::getline(std::cin, SNum);

		Num = atoi(SNum.c_str()); //convert string to int.

		if (Num < 4 || Num > 8)
		{
			std::cout << "Please pick a valid number." << std::endl;
			Num = 0;
		}
		else
		{
			std::cout << "Are you sure you want a " << Num << " letter word? (y/n)" << std::endl;
			FString Response = "";
			std::getline(std::cin, Response);//make sure the user picked the number they wanted.
			if (Response[0] == 'y' || Response[0] == 'Y')
			{
				break;
			}
			else
			{
				std::cout << "Pick a number from 4 to 8." << std::endl;
				Num = 0;
			}
		}

	} while (true);

	//grab a random word from the selected size.
	int32 RNum = rand() % ARRAY_SIZE + 0;
	switch (Num)
	{
	case 4:
		return FourLetters[RNum];
		break;
	case 5:
		return FiveLetters[RNum];
		break;
	case 6:
		return SixLetters[RNum];
		break;
	case 7:
		return SevenLetters[RNum];
		break;
	case 8:
		return EightLetters[RNum];
		break;
	default:
		return FiveLetters[RNum];
		break;
	}
	return FiveLetters[RNum];
}

//reset the game.
void FBullCowGame::Reset()
{
	HiddenWord = PickWordLength();
	CurrentGuess = 1;
	bGameWon = false;
	return;
}

//get max guesses based on length of the word.
int32 FBullCowGame::GetMaxGuesses() const
{
	TMap<int32, int32> WordLengthToMaxGuesses{ {4, 7}, {5, 10}, {6, 16}, {7, 20}, {8, 30} };

	return WordLengthToMaxGuesses[HiddenWord.length()];
}
int32 FBullCowGame::GetWordLength(FString Word) const { return Word.length(); }
int32 FBullCowGame::GetCurrentGuess() const { return CurrentGuess; }
FString FBullCowGame::GetHiddenWord() const { return HiddenWord; }
bool FBullCowGame::IsGameWon() const { return bGameWon; }

//checks if the guess is valid(if not isogram, if not all lowercase and if the wrong length.
EGuessStatus FBullCowGame::IsGuessValid(FString Guess) const
{
	if (!IsIsogram(Guess))//if guess isnt isogram
		return EGuessStatus::Not_Isogram;
	else if (!IsLowerCase(Guess))//if guess isnt all lowercase
		return EGuessStatus::Not_Lowercase;
	else if (GetWordLength(Guess) != GetWordLength(HiddenWord))//if guess length is wrong
		return EGuessStatus::Wrong_Length;
	else
		return EGuessStatus::OK;
}

//checks if guess is an isogram.
bool FBullCowGame::IsIsogram(FString Guess) const
{
	if (Guess.length() <= 1)//treat 0 and 1 letter words as isograms.
		return true;

	TMap<char, bool> LetterSeen;

	for (auto Letter : Guess)//itterate through letters of word(loop)
	{
		Letter = tolower(Letter);
		
		if (LetterSeen[Letter])//if the letter has already been added.
			return false;
		else
			LetterSeen[Letter] = true; //add the letter to the map.
	}

	return true;
}

//checks if guess is all lowercase.
bool FBullCowGame::IsLowerCase(FString Guess) const
{
	if (Guess.length() <= 1)//treat 0 and 1 letter words as lowercase.
		return true;

	for (auto Letter : Guess)//itterate through letters of word.
		if (!islower(Letter))//if letter isn't lowercase return false.
			return false;

	return true;
}

//once guess has been checked proceed with game
FBullCowCount FBullCowGame::SubmitValidGuess(FString Guess)
{
	CurrentGuess++;
	FBullCowCount BullCowCount;

	//loop through all letters in the Hidden Word.
	for (int32 i = 0; i < GetWordLength(HiddenWord); i++)
	{
		//compare letters of the guess against the hidden word.
		for (int32 j = 0; j < GetWordLength(Guess); j++)
		{
			//if they match
			if (HiddenWord[i] == Guess[j])
			{
				if (i == j)//if they're in the same place.
					BullCowCount.Bulls++;//increment bulls.
				else//if they're not.
					BullCowCount.Cows++;//increment cows.
			}
		}
	}

	if (BullCowCount.Bulls == GetWordLength(HiddenWord))//if bulls equals length of hidden word.
		bGameWon = true;
	else
		bGameWon = false;
	
	return BullCowCount;
}

//display if win or lose.
void FBullCowGame::PrintGameSummary()
{
	if (IsGameWon())
		std::cout << "WELL DONE! YOU WON!" << std::endl;
	else
		std::cout << "Better luck next time. The hidden word was " << HiddenWord << "." << std::endl;
}

