/*
The game logic (no view code or direct user interaction)
The game is a simple guess the word game based on mastermind.
*/

#pragma once
#include <string>
#define FString std::string
#define int32 int

struct FBullCowCount
{
	int32 Bulls = 0;
	int32 Cows = 0;
};

enum class EGuessStatus
{
	Invalid,
	OK,
	Not_Isogram,
	Wrong_Length,
	Not_Lowercase
};

class FBullCowGame
{
public:
	FBullCowGame();
	
	int32 GetMaxGuesses() const;
	int32 GetCurrentGuess() const;
	int32 GetWordLength(FString) const;
	FString GetHiddenWord() const;

	FString PickWordLength();
	
	EGuessStatus IsGuessValid(FString) const;
	bool IsGameWon() const;
	void PrintGameSummary();

	void Reset();
	
	//Counts bulls and cows, increases guess number.
	FBullCowCount SubmitValidGuess(FString);

private:
	static const int32 ARRAY_SIZE = 5;
	int32 CurrentGuess;
	FString HiddenWord;
	bool bGameWon;

	bool IsIsogram(FString) const;
	bool IsLowerCase(FString) const;

	FString FourLetters[ARRAY_SIZE] { "trim", "next", "paid", "trek", "pelt" };
	FString FiveLetters[ARRAY_SIZE] { "diary", "fairs", "grace", "years", "clone" };
	FString SixLetters[ARRAY_SIZE] { "corned", "hamlet", "bruise", "starch", "whites" };
	FString SevenLetters[ARRAY_SIZE] { "decorum", "moderns", "twirled", "dragons", "tumbled" };
	FString EightLetters[ARRAY_SIZE] { "adultery", "outlawed", "consumer", "counters", "relating" };
};