/*This is the console executable that makes use of the BullCow class
This acts as the view in a MVC pattern, and is responsible for all user
interaction. For game logic see the FBullCowGame class.
*/

#pragma once
#include <iostream>
#include <string>
#include "FBullCowGame.h"
#define FText std::string

void Introduction();
void PlayGame();

FBullCowGame BCGame;

FText GetValidGuess(FText Guess);
void PrintGuess(FText Guess);

bool AskToPlayAgain();

//Entry point of application.
int main()
{
	bool PlayAgain = false;
	do
	{
		Introduction();
		PlayGame();
		PlayAgain = AskToPlayAgain();
	} while (PlayAgain);
		
	return 0;
}

//introduce the game.
void Introduction()
{
	std::cout << std::endl << std::endl << "Welcome to Bulls and Cows! An amazingly boring word game!" << std::endl;
	std::cout << "          }   {         ___ " << std::endl;
	std::cout << "          (o o)        (o o) " << std::endl;
	std::cout << "   /-------\\ /          \\ /-------\\ " << std::endl; //I coppied :P
	std::cout << "  / | BULL |O            O| COW  | \\ " << std::endl;
	std::cout << " *  |-,--- |              |------|  * " << std::endl;
	std::cout << "    ^      ^              ^      ^ " << std::endl;
	
}

//Plays Game.
void PlayGame()
{
	BCGame.Reset();
	std::cout << "Can you guess the " << BCGame.GetWordLength(BCGame.GetHiddenWord()) << " letter isogram I'm thinking of?" << std::endl << std::endl;
	FText Guess = "";

	while (!BCGame.IsGameWon() && BCGame.GetCurrentGuess() <= BCGame.GetMaxGuesses())
	{
		Guess = GetValidGuess(Guess);

		FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);//submit valid guess to the game.
		PrintGuess(Guess);
		std::cout << "Bulls = " << BullCowCount.Bulls << ". Cows = " << BullCowCount.Cows << std::endl << std::endl;
	}

	BCGame.PrintGameSummary();
}

//loop continually until the user gives a valid guess.
FText GetValidGuess(FText Guess)
{
	EGuessStatus status = EGuessStatus::Invalid;
	do {
		std::cout << "Try " << BCGame.GetCurrentGuess() << " of " << BCGame.GetMaxGuesses() << ". Enter your guess: ";
		std::getline(std::cin, Guess);//get users input for a word.
		
		status = BCGame.IsGuessValid(Guess);//check if guess is valid

		//display what, if something is wrong with the guess.
		switch (status)
		{
		case EGuessStatus::Wrong_Length:
			std::cout << "Wrong Length! Please enter a " << BCGame.GetWordLength(BCGame.GetHiddenWord()) << " letter word." << std::endl;
			break;
		case EGuessStatus::Not_Isogram:
			std::cout << "Not an Isogram! Please enter a word with no repeating letters." << std::endl;
			break;
		case EGuessStatus::Not_Lowercase:
			std::cout << "Not Lowercase! Please only use lowercase letters." << std::endl;
			break;
		default:
			//breaks the loop is status is OK.
			break;
		}
		std::cout << std::endl;
	} while (status != EGuessStatus::OK);
	return Guess;
}

//repeat the guess back to them.
void PrintGuess(FText Guess)
{
	std::cout << "Your Guess is " << Guess << std::endl;
}

//Ask the user to play again.
bool AskToPlayAgain()
{
	std::cout << "Do you want to play again? (y/n)" << std::endl;
	FText Response = "";
	std::getline(std::cin, Response);

	return (Response[0] == 'y' || Response[0] == 'Y');
}
